@@ -42,6 +42,7 @@ class PostExtractor:
     link_regex = re.compile(r"href=\"https:\/\/lm\.facebook\.com\/l\.php\?u=(.+?)\&amp;h=")
 
     photo_link = re.compile(r'href=\"(/[^\"]+/photos/[^\"]+?)\"')
+    photo_link_2 = re.compile(r'href=\"(/photo.php[^\"]+?)\"')
     image_regex = re.compile(
         r'<a href=\"([^\"]+?)\" target=\"_blank\" class=\"sec\">View Full Size<\/a>',
         re.IGNORECASE,
@@ -68,6 +69,7 @@ class PostExtractor:
 
         self._data_ft = None
 
+    # TODO: This is getting ugly, create a dataclass for Post
     def make_new_post(self) -> Post:
         return {
             'post_id': None,
@@ -76,6 +78,7 @@ class PostExtractor:
             'shared_text': None,
             'time': None,
             'image': None,
+            'images': None,
             'video': None,
             'video_thumbnail': None,
             'video_id': None,
@@ -94,6 +97,10 @@ class PostExtractor:
             'shared_user_id': None,
             'shared_username': None,
             'shared_post_url': None,
+            'available': None,
+            'comments_full': None,
+            'reactors': None,
+            'w3_fb_url': None,
         }
 
     def extract_post(self) -> Post:
@@ -117,6 +124,7 @@ class PostExtractor:
             self.extract_is_live,
             self.extract_factcheck,
             self.extract_share_information,
+            self.extract_availability,
         ]
 
         post = self.make_new_post()
@@ -140,7 +148,7 @@ class PostExtractor:
             except Exception as ex:
                 log_warning("Exception while running %s: %r", method.__name__, ex)
 
-        if self.options.get('reactions'):
+        if self.options.get('reactions') or self.options.get('reactors'):
             try:
                 reactions = self.extract_reactions()
             except Exception as ex:
@@ -152,10 +160,17 @@ class PostExtractor:
             else:
                 post.update(reactions)
 
+        if self.options.get('comments'):
+            try:
+                comments = self.extract_comments_full()
+                post.update(comments)
+            except Exception as ex:
+                log_warning("Exception while extracting comments: %r", ex)
+
         return post
 
     def extract_post_id(self) -> PartialPost:
-        return {'post_id': self.data_ft.get('mf_story_key')}
+        return {'post_id': self.data_ft.get('top_level_post_id')}
 
     def extract_username(self) -> PartialPost:
         username = self.element.find('h3 strong a')
@@ -175,8 +190,6 @@ class PostExtractor:
             if match:
                 url = utils.urljoin(FB_MOBILE_BASE_URL, match.groups()[0].replace("&amp;", "&"))
                 response = self.request(url)
-                # If .story_body_container is commented the find function won't work
-                response.html.html = response.html.html.replace('<!--', '').replace('-->', '')
                 element = response.html.find('.story_body_container', first=True)
 
         nodes = element.find('p, header, span[role=presentation]')
@@ -252,7 +265,7 @@ class PostExtractor:
 
     def extract_image(self) -> PartialPost:
         image_link = self.extract_photo_link()
-        if image_link is not None:
+        if image_link["image"] is not None:
             return image_link
         return self.extract_image_lq()
 
@@ -267,10 +280,14 @@ class PostExtractor:
             if image_container is None:
                 continue
 
+            src = image_container.attrs.get('src')
+            if src:
+                return {'image': src, 'images': [src]}
             style = image_container.attrs.get('style', '')
             match = self.image_regex_lq.search(style)
             if match:
-                return {'image': utils.decode_css_url(match.groups()[0])}
+                src = utils.decode_css_url(match.groups()[0])
+                return {'image': src, 'images': [src]}
 
         return None
 
@@ -343,7 +360,9 @@ class PostExtractor:
 
     def extract_photo_link(self) -> PartialPost:
         images = []
-        matches = self.photo_link.finditer(self.element.html)
+        matches = list(self.photo_link.finditer(self.element.html))
+        if not matches:
+            matches = self.photo_link_2.finditer(self.element.html)
 
         for match in matches:
             url = utils.urljoin(FB_MOBILE_BASE_URL, match.groups()[0])
@@ -352,7 +371,10 @@ class PostExtractor:
             html = response.text
             match = self.image_regex.search(html)
             if match:
-                images.append(match.groups()[0].replace("&amp;", "&"))
+                url = match.groups()[0].replace("&amp;", "&")
+                if not url.startswith("http"):
+                    url = utils.urljoin(FB_MOBILE_BASE_URL, url)
+                images.append(url)
         image = images[0] if images else None
         return {"image": image, "images": images}
 
@@ -373,9 +395,74 @@ class PostExtractor:
         """
         url = self.post.get('post_url')
         post_id = self.post.get('post_id')
+        w3_fb_url = url and utils.urlparse(url)._replace(netloc='www.facebook.com').geturl()
+
+        reaction_url = f'https://m.facebook.com/ufi/reaction/profile/browser/?ft_ent_identifier={post_id}'
+        response = self.request(reaction_url)
+        reactions = {}
+
+        # Dict mapping class names to human readable reaction names. Prepopulated in case FB doesn't include them
+        reaction_lookup = {
+            'sx_cbd149': 'Like',
+            'sx_202991': 'Love',
+            'sx_41edbc': 'Care',
+            'sx_0d839a': 'Haha',
+            'sx_2b1a8e': 'Wow',
+            'sx_454e38': 'Angry',
+            'sx_1a0b4b': 'Sad'
+        }
+
+        for span in response.html.find("span[aria-label]"):
+            label = span.attrs.get("aria-label", "")
+            if " people reacted with " in label:
+                reaction_count, reaction_type = label.split(" people reacted with ")
+                reactions[reaction_type.lower()] = utils.convert_numeric_abbr(reaction_count)
+                if self.options.get("reactors"):
+                    emoji_class = span.find("i", first=True).attrs.get("class")[-1]
+                    reaction_lookup[emoji_class] = reaction_type
+
+        reactors = []
+
+        if self.options.get("reactors"):
+            """Fetch people reacting to an existing post obtained by `get_posts`.
+            Note that this method may raise one more http request per post to get all reactors"""
+            logger.debug("Fetching reactors")
+            elems = list(response.html.find("div#reaction_profile_browser>div"))
+            more = response.html.find("div#reaction_profile_pager a", first=True)
+            if more:
+                url = utils.urljoin(FB_MOBILE_BASE_URL, more.attrs.get("href"))
+                url = url.replace("limit=50", f"limit={1e6}")
+                logger.debug(f"Fetching {url}")
+                response = self.request(url)
+                prefix_length = len('for (;;);')
+                data = json.loads(response.text[prefix_length:])  # Strip 'for (;;);'
+
+                for action in data['payload']['actions']:
+                    if action['cmd'] == 'append':
+                        html = utils.make_html_element(f"<div id='reaction_profile_browser'>{action['html']}</div>", url=FB_MOBILE_BASE_URL)
+                        more_elems = html.find('div#reaction_profile_browser>div')
+                        elems.extend(more_elems)
+            logger.debug(f"Found {len(elems)} reactors")
+            for elem in elems:
+                emoji_class = elem.find("div>i:not(.nub)", first=True).attrs.get("class")[-1]
+                if not reaction_lookup.get(emoji_class):
+                    logger.error(f"Don't know {emoji_class}")
+                reactors.append({
+                    "name": elem.find("strong", first=True).text,
+                    "link": utils.urljoin(FB_BASE_URL, elem.find("a", first=True).attrs.get("href")),
+                    "type": reaction_lookup.get(emoji_class)
+                })
+
+        if reactions:
+            return {
+                'likes': reactions.get("like"),
+                'reactions': reactions,
+                'reactors': reactors,
+                'fetched_time': datetime.now(),
+                'w3_fb_url': w3_fb_url,
+            }
 
         if url:
-            w3_fb_url = utils.urlparse(url)._replace(netloc='www.facebook.com').geturl()
             resp = self.request(w3_fb_url)
             for item in self.parse_share_and_reactions(resp.text):
                 data = item['jsmods']['pre_display_requires'][0][3][1]['__bbox']['result'][
@@ -408,7 +495,7 @@ class PostExtractor:
     def extract_video_lowres(self, video_data_element):
         try:
             data = json.loads(video_data_element.attrs['data-store'])
-            return {'video': data.get('src')}
+            return {'video': data.get('src').replace("\\/", "/")}
         except JSONDecodeError as ex:
             logger.error("Error parsing data-store JSON: %r", ex)
         except KeyError:
@@ -483,9 +570,13 @@ class PostExtractor:
     def extract_share_information(self):
         if not self.data_ft.get("original_content_id"):
             return None
-        logger.debug("%s is a share of %s", self.post["post_id"], self.data_ft["original_content_id"])
+        logger.debug(
+            "%s is a share of %s", self.post["post_id"], self.data_ft["original_content_id"]
+        )
         # A shared post contains an <article> element within it's own <article> element, or a header element for a shared image
-        raw_post = self.element.find("article article, .story_body_container .story_body_container header", first=True)
+        raw_post = self.element.find(
+            "article article, .story_body_container .story_body_container header", first=True
+        )
         # We can re-use the existing parsers, as a one level deep recursion
         shared_post = PostExtractor(raw_post, self.options, self.request)
         return {
@@ -493,9 +584,68 @@ class PostExtractor:
             'shared_time': shared_post.extract_time().get("time"),
             'shared_user_id': self.data_ft["original_content_owner_id"],
             'shared_username': shared_post.extract_username().get("username"),
-            'shared_post_url': shared_post.extract_post_url().get("post_url")
+            'shared_post_url': shared_post.extract_post_url().get("post_url"),
         }
 
+    def extract_availability(self):
+        return {
+            'available': ">This content isn't available at the moment<" not in self.element.html
+        }
+
+    def extract_comments_full(self):
+        """Fetch comments for an existing post obtained by `get_posts`.
+        Note that this method may raise multiple http requests per post to get all comments"""
+        url = self.post.get('post_url').replace(FB_BASE_URL, FB_MOBILE_BASE_URL)
+        logger.debug(f"Fetching {url}")
+        response = self.request(url)
+        comments = list(response.html.find('div[data-sigil="comment"]'))
+        more = response.html.find("a", containing="View more comments", first=True)
+        while more:
+            url = utils.urljoin(FB_MOBILE_BASE_URL, more.attrs.get("href"))
+            logger.debug(f"Fetching {url}")
+            response = self.request(url)
+            more_comments = response.html.find('div[data-sigil="comment"]')
+            comments.extend(more_comments)
+            more = response.html.find("a", containing="View more comments", first=True)
+        logger.debug(f"Found {len(comments)} comments")
+        result = []
+        for comment in comments:
+            comment_id = comment.attrs.get("id")
+            first_link = comment.find("div:not([data-sigil])>a[href]:not([data-click]):not([data-store]):not([data-sigil]):not([class])", first=True)
+            comment_body_elem = comment.find('[data-sigil="comment-body"]', first=True)
+            commenter_meta = None
+            if first_link:
+                url = utils.urljoin(FB_BASE_URL, first_link.attrs.get("href"))
+                name = first_link.text
+                if "\n" in name:
+                    commenter_meta, name = name.split("\n")
+            else:
+                # Adjacent div to comment body, if not logged in. No link to user page available in that case
+                url = None
+                elem = comment_body_elem.element.getprevious()
+                if elem is not None:
+                    name = elem.text
+                else:
+                    name = None
+
+            text = comment_body_elem.text
+            # Try to extract from the abbr element
+            date_element = comment.find('abbr', first=True)
+            if date_element:
+                date = utils.parse_datetime(date_element.text, search=False)
+            else:
+                date = None
+
+            result.append({
+                "comment_id": comment_id,
+                "commenter_url": url,
+                "commenter_name": name,
+                "commenter_meta": commenter_meta,
+                "comment_text": text,
+                "comment_time": date,
+            })
+        return {"comments_full": result}
+
     def parse_share_and_reactions(self, html: str):
         bad_jsons = self.shares_and_reactions_regex.findall(html)
         for bad_json in bad_jsons:
